// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Message types that are passed through the Omnibus.
enum OmnibusMessageType { event, request, response }

/// Omnibus message base
///
/// An omnibus message has an [omnibusId] so it can be safely identified by as a
/// specific message type. All message data is stored in the [data] Map. The map
/// should only contain JSON compatible data. A map was choosen to allow easy
/// message filtering and easy JSON conversion (as required for integrating
/// non-Dart handlers).
abstract class OmnibusMessage {
  /// Message identifier key
  ///
  /// Use the following convention for this identifier:
  /// `package_name.MessageName`
  final String omnibusId;

  /// Message type
  final OmnibusMessageType type;

  /// Default constuctor
  OmnibusMessage(this.omnibusId, this.type);

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds();
}

/// Event message
///
/// An event message is published when the module wants to notify other handlers
/// of what is going on. An example of this is a progress update. This message
/// can be used for publish-subscribe messaging.
abstract class OmnibusEvent extends OmnibusMessage {
  /// Default constructor
  OmnibusEvent(String omnibusId) : super(omnibusId, OmnibusMessageType.event);
}

/// Request message
///
/// A request message is published when the module requires certain data. This
/// message can be used for request-reply messasing.
abstract class OmnibusRequest extends OmnibusMessage {
  /// Default constructor
  OmnibusRequest(String omnibusId)
      : super(omnibusId, OmnibusMessageType.request);
}

/// Response message
///
/// The response message is returned by handlers in order to reply to a request
/// message.
abstract class OmnibusResponse extends OmnibusMessage {
  /// Default constructor
  OmnibusResponse(String omnibusId)
      : super(omnibusId, OmnibusMessageType.response);
}
