// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Message handler state information
///
/// This class should be extended by the encapsulating module so it can store
/// all state information required by the handlers in the module. It can be
/// shared between handlers or only be used by one handler. It can also contain
/// a reference to the parent bus and the shared HTTP client.
abstract class StateData {}

/// Event callback typedef
typedef Future<Null> OmnibusEventCallback<T>(T message, StateData state);

/// Request callback typedef
typedef Future<OmnibusResponse> OmnibusRequestCallback<T>(
    T message, StateData state);

/// Handler semantics
class OmnibusHandler {
  /// Type (event or request channel)
  final OmnibusMessageType type;

  /// Target omnibusId
  final String omnibusId;

  /// State data
  final StateData state;

  /// Constuctor
  OmnibusHandler(this.type, this.omnibusId, this.state);
}

/// Event handler base
abstract class OmnibusEventHandler extends OmnibusHandler {
  /// Constructor
  OmnibusEventHandler(String omnibusId, StateData state)
      : super(OmnibusMessageType.event, omnibusId, state);

  /// Process incoming message.
  Future<List<Future<Null>>> process(OmnibusEvent data);
}

/// Request handler base
abstract class OmnibusRequestHandler extends OmnibusHandler {
  /// Constructor
  OmnibusRequestHandler(String omnibusId, StateData state)
      : super(OmnibusMessageType.request, omnibusId, state);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data);
}
