// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Omnibus responses helper
class OmnibusResponses {
  /// Evaluated responses
  List<OmnibusResponse> _completed = new List<OmnibusResponse>();

  /// Future responses
  List<Future<OmnibusResponse>> _futures;

  /// Completer that completes when all future responses are received.
  Completer<List<OmnibusResponse>> _completer =
      new Completer<List<OmnibusResponse>>();

  /// Constructor
  OmnibusResponses(this._futures) {
    if (_futures.length == 0) {
      // Complete now if there are no futures.
      _completer.complete([]);
    } else {
      // Set completion callbacks.
      for (final future in _futures) {
        future.then((OmnibusResponse returnValue) {
          _completed.add(returnValue);
          _futures.remove(future);

          // Complete _completer if there are no future responses left.
          if (_futures.isEmpty) {
            _completer.complete(_completed);
          }
        });
      }
    }
  }

  /// Total number of responses
  int get count => _completed.length + _futures.length;

  /// Await the evaluation of all processes
  ///
  /// Returns the completer future.
  Future<List<OmnibusResponse>> onComplete() {
    return _completer.future;
  }

  /// Pass all responses with the given omnibusId to a new Stream.
  Stream stream(String omnibusId, dynamic converter(OmnibusResponse input)) {
    var controller = new StreamController();

    // Add already evaluated responses.
    for (final response in _completed) {
      if (response.omnibusId == omnibusId) {
        controller.add(converter(response));
      }
    }

    // Set callback for future responses.
    for (final futureResponse in _futures) {
      futureResponse.then((OmnibusResponse response) {
        // Note that the standard response completion logic is already handled
        // in the constructor.
        if (response.omnibusId == omnibusId) {
          controller.add(converter(response));
        }
      });
    }

    // Close stream when the completer is ready.
    _completer.future.then((_) => controller.close());

    return controller.stream;
  }
}
