// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.event_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';

import 'messages_lib.dart' as v0;

/// This variable is used to check that events are indeed processed
/// asynchronously.
var asyncSwitch = true;

void main() {
  test('Send Email message', () async {
    var bus = new Omnibus();
    bus.bind(new v0.EmailHandler(emailHandler, new EmptyState(),
        receiver: 'me@example.com'));

    // Process asynchronous (publish using .event alias).
    bus.event(new v0.Email('Hi!',
        sender: 'sender@example.com', receiver: 'notme@example.com'));
    bus.event(new v0.Email('Hi!',
        sender: 'sender@example.com', receiver: 'me@example.com'));

    var processes = (await bus.publishEvent(new v0.Email('Hi!',
            sender: 'sender@example.com', receiver: 'me@example.com')))
        .length;
    // There should be one event handler that processed this event
    // (emailHandler)
    expect(processes, equals(1));
  });
}

Future<Null> emailHandler(v0.Email email, StateData state) async {
  expect(email.receiver, equals('me@example.com'));
}
