// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.global_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';

import 'messages_lib.dart' as v0;

void main() {
  test('Test global handlers', () async {
    var bus = new Omnibus();

    // Bind global handlers.
    var globalEventHandler = new GlobalEventHandler();
    var globalRequestHandler = new GlobalRequestHandler();
    bus.bind(globalEventHandler);
    bus.bind(globalRequestHandler);

    // Run some events.
    bus.publishEvent(new v0.MessageForAll('This message is for everyone!'));
    bus.publishEvent(
        new v0.Email('Hi!', sender: 'a@here.com', receiver: 'b@here.com'));

    // Run some requests.
    bus.publishRequest(
        new v0.CalculationRequest(1.0, 1.0, v0.CalculationOperator.subtract));
    bus.publishRequest(
        new v0.CalculationRequest(1.0, 1.0, v0.CalculationOperator.multiply));

    // This request goes through all json_map functions.
    bus.publishRequest(new v0.FakeRequest({
      'numbers': [0, 1, 2, 3]
    }, {
      'booleans': [true, false, true]
    }));

    // Wait for async completion.
    new Future.delayed(new Duration(seconds: 1), () {
      expect(globalEventHandler.totalEvents, equals(2));
      expect(globalRequestHandler.totalRequests, equals(3));
    });
  });
}

/// Event handler that receives all events.
class GlobalEventHandler extends OmnibusEventHandler {
  /// Counted events.
  int totalEvents = 0;

  /// Constructor
  GlobalEventHandler() : super(omnibusGlobalOmnibusId, new EmptyState());

  /// Receive event.
  Future<List<Future<Null>>> process(OmnibusEvent event) async {
    totalEvents++;
    return [];
  }
}

/// Request handler that receives all requests.
class GlobalRequestHandler extends OmnibusRequestHandler {
  /// Counted events.
  int totalRequests = 0;

  /// Constructor
  GlobalRequestHandler() : super(omnibusGlobalOmnibusId, new EmptyState());

  /// Receive event.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest event) async {
    totalRequests++;
    return [];
  }
}
